/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/* Copyright (c) 2004-2005 Extreme! Lab, Indiana University. All rights reserved.
 * This software is open source. See the bottom of this file for the licence.
 * $Id: WsdlDefinitions.java,v 1.3 2006/04/26 23:22:18 aslom Exp $ */
package xsul5.wsdl;

import java.util.Iterator;

import javax.xml.namespace.QName;

import org.xmlpull.infoset.XmlAttribute;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlElementView;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.XmlNamespace;
import org.xmlpull.infoset.view.XmlValidationException;
import xsul5.MLogger;
import xsul5.XmlConstants;
import xsul5.XmlValidation;



/**
 * Implementation of <a href="http://www.w3.org/TR/wsdl#_message">WSLD 1.1 Definitions</a>.
 * <br/>Overview of <a href="http://www.w3.org/TR/wsdl#_document-s">WSDL structure</a>
 *
 * @version $Revision: 1.3 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class WsdlDefinitions extends WsdlXmlViewBase implements XmlValidation {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    private final static String NAME = "definitions";
    
    //public final static XmlNamespace TYPE = builder.newNamespace("http://schemas.xmlsoap.org/wsdl/");
    
    public WsdlDefinitions(String targetNamespace) {
        super(NAME);
        setTargetNamespace(targetNamespace);
    }
    
    public WsdlDefinitions(XmlElement target) {
        super(target.getName(), target);
    }
    
    
    /**
     * Try to extract wsdl:definitions/wsdl:types or null if it does not exist.
     */
    public XmlElement getTypes() {
        return xml().element(NS, "types");
    }
    
    /**
     * Try to extract wsdl:definitions/wsdl:types or create new one if it does not exist.
     */
    public XmlElement getOrCreateTypes() {
        return xml().element(NS, "types", true);
    }
    
    public XmlElement findXsSchemaInTypesWithTargetNamespace(String schemaTargetNamespace) {
        XmlElement types = getTypes();
        if (types == null) {
            return null;
        }
        for (Iterator si = types.elements(XmlConstants.XS_NS, "schema").iterator(); si.hasNext();) {
            XmlElement schema = (XmlElement) si.next();
            String targetNs = schema.attributeValue("targetNamespace");
            if (schemaTargetNamespace.equals(targetNs)) {
                return schema;
            }
        }
        return null;
    }
    
    
    public String getTargetNamespace() {
        return xml().requiredAttributeValue("targetNamespace");
    }
    
    public void setTargetNamespace(String targetNamespace) {
        XmlAttribute att = xml().attribute("targetNamespace");
        if (att != null) {
            xml().removeAttribute(att);
        }
        xml().setAttributeValue("targetNamespace", targetNamespace);
    }
    
    public Iterable<WsdlMessage> messages() {
        return xml().elements(NS, WsdlMessage.TYPE_NAME, WsdlMessage.class);
    }
    
    public WsdlMessage getMessage(String messageName) {
        for (WsdlMessage message : messages()) {
            if (message.getName().equals(messageName)) {
                return message;
            }
        }
        return null;
    }
    
    public WsdlMessage addMessage(String messageName) throws XmlValidationException {
        if(getMessage(messageName) != null) {
            throw new XmlValidationException("there is already message "+messageName);
        }
        WsdlMessage msg = new WsdlMessage(messageName);
        xml().addElement(msg.xml());
        return msg;
    }


    
    public Iterable<WsdlPortType> portTypes() {
        return xml().elements(NS, WsdlPortType.TYPE_NAME, WsdlPortType.class);
    }

    public WsdlPortType getPortType(String portTypeName) {
        for (WsdlPortType possiblePT : portTypes()) {
            if(possiblePT.getName().equals(portTypeName)) {
                return possiblePT;
            }
        }
////        throw new WsdlException(
////            "PortType '"+portTypeName+ "' is not available and no alternative can be found in "+toString());
        return null;
    }
    
    public WsdlPortType addPortType(String portTypeName) {
        if(getPortType(portTypeName) != null) {
            throw new XmlValidationException("there is already portType "+portTypeName);
        }
        WsdlPortType pt = new WsdlPortType(portTypeName);
        xml().addElement(pt.xml());
        return pt;
    }
    
    public Iterable<WsdlBinding> bindings() {
        return xml().elements(NS, WsdlBinding.TYPE_NAME, WsdlBinding.class);
    }
    
    public WsdlBinding getBinding(String bindingName) {
        for(WsdlBinding bndng : bindings()) {
            if(bndng.getName().equals(bindingName)) {
                return bndng;
            }
        }
        return null;
    }

    public WsdlBinding addBinding(String name, QName portTypeQName) {
        if(getBinding(name) != null) {
            throw new XmlValidationException("there is already binding "+name);
        }
        WsdlBinding bi = new WsdlBinding(name, portTypeQName);
        xml().addElement(bi.xml());
        return bi;
    }
    
    public WsdlBinding addBinding(WsdlBinding binding) {
        if (binding == null) {
            new IllegalArgumentException();
        }
        xml().addElement(binding.xml());
        return binding;
    }
    
    public Iterable<WsdlService> services() {
        return xml().elements(NS, WsdlService.NAME, WsdlService.class);
    }
    
    public WsdlService getService(String serviceName) {
        for(WsdlService serv : services()) {
            if(serv.getName().equals(serviceName)) {
                return serv;
            }
        }
        return null;
    }

    public WsdlService addService(String name) {
        if(getService(name) != null) {
            throw new XmlValidationException("there is already service "+name);
        }
        WsdlService serv = new WsdlService(name);
        xml().addElement(serv.xml());
        return serv;
    }
    
    public void xmlValidateData() throws XmlValidationException {
        if (!NS.equals(xml().getNamespace())) {
            throw new XmlValidationException(
                "expected element to be in namespace " + NS.getName()
                + " not " + xml().getNamespace().getName());
        }
        if (!NAME.equals(xml().getName())) {
            throw new XmlValidationException(
                "expected element to have name " + NAME + " not " + xml().getName());
        }
        if (getTargetNamespace() == null) {
            throw new XmlValidationException("targetNamespace is required");
        }
        for (Iterator<WsdlMessage> iter = messages().iterator(); iter.hasNext();) {
            iter.next();
        }
        for (WsdlPortType pt : portTypes()) {
            pt.xmlValidate();
        }
        for (WsdlBinding bd : bindings()) {
            bd.xmlValidate();
        }
        for (WsdlService sv : services()) {
            sv.xmlValidate();
        }
    }
    
    
    public String toString() {
        return builder.serializeToString(this) ;
    }
    
    
    //    private static class IteratorAdapter implements Iterator {
    //      final Iterator iter;
    //      IteratorAdapter(Iterator iter) {
    //          this.iter = iter;
    //      }
    //      public boolean hasNext() {
    //          return iter.hasNext();
    //      }
    //      public Object next() {
    //          iter.next();
    //      }
    //      public void remove() {
    //          iter.remove();
    //      }
    //
    //    }
    
}



//// typical hockey-pockey exchange of control
//class IterableAdapter<T> implements Iterable<T> {
//    private final Iterable iterable;
//
//    IterableAdapter(Iterable iterable) {
//        this.iterable = iterable;
//    }
//
//    public T next(Iterator iter) {
//        return (T)iter.next();
//    }
//
//    public Iterator<T> iterator() {
//        final Iterator iter = iterable.iterator();
//        return new Iterator<T>() {
//
//            public boolean hasNext() {
//                return iter.hasNext();
//            }
//
//            public void remove() {
//                iter.remove();
//            }
//
//            public T next() {
//                return IterableAdapter.this.next(iter);
//            }
//
//        };
//    }
//}


/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (C) 2004-2005 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/

