/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: WsdlMessage.java,v 1.1 2005/11/14 19:31:25 aslom Exp $
 */

package xsul5.wsdl;

import javax.xml.namespace.QName;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.view.XmlValidationException;
import xsul5.MLogger;
import xsul5.XmlConstants;
import xsul5.wsdl.WsdlDefinitions;
import xsul5.wsdl.WsdlXmlViewBase;



/**
 * Implementation of <a href="http://www.w3.org/TR/wsdl#_message">WSDL 1.1 message element</a>
 * <br/><pre><a href="http://www.w3.org/TR/wsdl#_document-s">&lt;wsdl:message name="nmtoken">
 *  &lt;wsdl:documentation .... />?
 *  &lt;part name="nmtoken" element="qname"? type="qname"?/> *
 * &lt;/wsdl:message></a></pre>
 *
 * @version $Revision: 1.1 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class WsdlMessage extends WsdlXmlViewBase {
    //private final static XmlNamespace TYPE_NS = NS;
    public final static String TYPE_NAME = "message";
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    
    public WsdlMessage(String name) {
        super(TYPE_NAME);
        setName(name);
    }
    
    public WsdlMessage(XmlElement target) {
        super(TYPE_NAME, target);
    }
    
    public WsdlDefinitions getDefinitions() {
        XmlElement defsEl = (XmlElement)xml().getParent();
        return defsEl.viewAs(WsdlDefinitions.class);
    }
    
//    public String getName() {
//        return xml().attributeValue("name");
//    }
    
    public void setName(String partName) throws XmlValidationException {
        xmlSetRequiredAttributeNcNameValue("name", partName);
    }
    
    public String getName() throws XmlValidationException {
        return xmlRequireAttributeNcNameValue("name");
    }
    
    /**
     * @deprecated
     */
    public String getMessageName() {
        return getName();
    }
    
    public Iterable<WsdlMessagePart> parts() {
        return xml().elements(WsdlDefinitions.NS, WsdlMessagePart.NAME, WsdlMessagePart.class);
    }
    
    public WsdlMessage addPart(WsdlMessagePart part) {
        xml().addElement(part.xml());
        return this;
    }
    
    public WsdlMessage addPartWithElement(String name, QName partElement) {
        WsdlMessagePart part = new WsdlMessagePart(name);
        part.setElement(partElement);
        xml().addElement(part.xml());
        return this;
    }
    
    // discouraged from use as we try to encourage doc/literal style ...
//    public WsdlMessage addPartWithType(String name, QName partType) {
//        WsdlMessagePart part = new WsdlMessagePart(name);
//        part.setType(partType);
//        xml().addElement(part.xml());
//        return this;
//    }
    
    public void xmlValidate() throws XmlValidationException {
        super.xmlValidate();
//        if (!TYPE_NS.equals(xml().getNamespace())) {
//            throw new XmlValidationException(
//                "expected element to be in namespace " + TYPE_NS.getName()
//                + " not " + xml().getNamespaceName());
//        }
//        if (!TYPE_NAME.equals(xml().getName())) {
//            throw new XmlValidationException(
//                "expected element to have name '" + TYPE_NAME
//                + "' not '" + xml().getName() + "'");
//        }
        getDefinitions();
        getName();
//        if (getName() == null) {
//            throw new XmlValidationException(
//                "message name attributes is required");
//        }
        for (WsdlMessagePart part: parts()) {
            part.xmlValidate();
        }
    }
    
    public String toString() {
        return builder.serializeToString(this) ;
    }
}


/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (C) 2004-2005 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/





