/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: WsdlMessagePart.java,v 1.1 2005/11/14 19:31:25 aslom Exp $
 */

package xsul5.wsdl;

import javax.xml.namespace.QName;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.view.XmlValidationException;
import xsul5.MLogger;
import xsul5.XmlConstants;
import xsul5.XmlValidation;
import xsul5.wsdl.WsdlDefinitions;
import xsul5.wsdl.WsdlMessage;
import xsul5.wsdl.WsdlXmlViewBase;
//import xsul.util.Util;

/**
 * Implementation of <a href="http://www.w3.org/TR/wsdl#_message">WSDL 1.1 message/part</a>
 * <br/><a href="http://www.w3.org/TR/wsdl#_document-s">&lt;part
 *   name="nmtoken" element="qname"? type="qname"?/></a>
 *
 * @version $Revision: 1.1 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class WsdlMessagePart extends WsdlXmlViewBase {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    public final static String NAME = "part";
    
    
    public WsdlMessagePart(String name) {
        super(NAME);
        setName(name);
    }
    
    public WsdlMessagePart(XmlElement target) {
        super(NAME, target);
    }
    
    public WsdlMessage getMessage() {
        XmlElement parentEl = (XmlElement)xml().getParent();
        return parentEl.viewAs(WsdlMessage.class);
    }
    
    public WsdlDefinitions getDefinitions() {
        return getMessage().getDefinitions();
    }
    
    public void setName(String partName) throws XmlValidationException {
//        if(partName == null) throw new XmlValidationException("part name can not be null");
//        xmlSetAttributeNcNameValue("name", partName);
        xmlSetRequiredAttributeNcNameValue("name", partName);
    }
    
    public String getName() throws XmlValidationException {
        return xmlRequireAttributeNcNameValue("name");
        //String b = xml().attributeValue("name");
//        String b = xmlRequireAttributeNcNameValue("name");
//        if (b == null) {
//            throw new XmlValidationException(
//                "missing required name on message part in message " + getMessage());
//        }
        //return b;
    }

    public void setElement(QName partElement) throws XmlValidationException {
        xmlSetAttributeQNameValue("element", partElement);
    }
        
    public QName getElement() throws XmlValidationException {
        return xmlGetAttributeQNameValue("element");
//      String b = xml().attributeValue("element");
//        if(b == null) {
//            return null;
//        }
        //return Util.toQName(this, b);
    }

    public void setType(QName partType) throws XmlValidationException {
        xmlSetAttributeQNameValue("type", partType);
    }
    
    public QName getType() throws XmlValidationException {
        return xmlGetAttributeQNameValue("type");
//        String b = xml().attributeValue("type");
//        if (b == null) {
//            return null;
//        }
        //return Util.toQName(this, b);
    }
    
    public void xmlValidate() throws XmlValidationException {
        super.xmlValidate();
        getName();
        getElement();
        getType();
        getMessage();
        getDefinitions();
    }
    
}


/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (C) 2004 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/





