/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: WsdlPortTypeOperation.java,v 1.1 2005/11/14 19:31:25 aslom Exp $
 */

package xsul5.wsdl;

import javax.xml.namespace.QName;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.XmlNamespace;
import org.xmlpull.infoset.view.XmlValidationException;
import xsul5.MLogger;
import xsul5.XmlConstants;



/**
 * Implementation of WSLD 1.1 operation
 *
 * @version $Revision: 1.1 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class WsdlPortTypeOperation extends WsdlXmlViewBase {
    
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    public final static String NAME = "operation";
    
    
    public WsdlPortTypeOperation(String operationName) {
        super(NAME);
        //validateData();
        setName(operationName);
    }
    
    public WsdlPortTypeOperation(XmlElement target) {
        super(NAME, target);
        //validateData();
    }
    
    public WsdlDefinitions getDefinitions() {
        return getPortType().getDefinitions();
    }
    
    public WsdlPortType getPortType() {
        XmlElement portTypeEl = (XmlElement)xml().getParent();
        return portTypeEl.viewAs(WsdlPortType.class);
    }
    
    public void setName(String partName) throws XmlValidationException {
        xmlSetRequiredAttributeNcNameValue("name", partName);
    }
    
    public String getName() throws XmlValidationException {
        return xmlRequireAttributeNcNameValue("name");
    }
    
    public String getOperationName() {
        //return xml().attributeValue("name");
        return getName();
    }
    
    
    public WsdlPortTypeInput getInput() {
        XmlElement inputEl = xml().element(WsdlDefinitions.NS, "input");
        if (inputEl == null) {
            return null;
        }
        return inputEl.viewAs(WsdlPortTypeInput.class);
    }
    
    public WsdlPortTypeOperation setInput(WsdlPortTypeInput input) {
        xml().replaceLikeElementsWith(input.xml());
        return this;
    }
    
    public WsdlPortTypeOperation setInput(WsdlMessage messageRef) throws XmlValidationException {
        QName q = buildQnameForMessage(messageRef);
        return setInput(new WsdlPortTypeInput(q));
    }

    public WsdlPortTypeOperation setInput(String name, WsdlMessage messageRef) throws XmlValidationException {
        QName q = buildQnameForMessage(messageRef);
        return setInput(new WsdlPortTypeInput(name, q));
    }
    
    public WsdlPortTypeOutput getOutput() {
        XmlElement inputEl = xml().element(WsdlDefinitions.NS, "output");
        if (inputEl == null) {
            return null;
        }
        return inputEl.viewAs(WsdlPortTypeOutput.class);
    }
    
    public WsdlPortTypeOperation setOutput(WsdlPortTypeOutput output) {
        xml().replaceLikeElementsWith(output.xml());
        return this;
    }
    
    public WsdlPortTypeOperation setOutput(WsdlMessage messageRef) throws XmlValidationException {
        QName q = buildQnameForMessage(messageRef);
        return setOutput(new WsdlPortTypeOutput(q));
    }

    public WsdlPortTypeOperation setOutput(String name, WsdlMessage messageRef) throws XmlValidationException {
        QName q = buildQnameForMessage(messageRef);
        return setOutput(new WsdlPortTypeOutput(name, q));
    }

    public Iterable<WsdlPortTypeFault> faults() throws XmlValidationException {
        return xml().elements(WsdlPortTypeFault.NS, WsdlPortTypeFault.NAME, WsdlPortTypeFault.class);
    }

    public WsdlPortTypeOperation addFault(WsdlPortTypeFault fault) {
        //TODO: check for dulicate fault@name
        xml().addElement(fault.xml());
        return this;
    }

    public WsdlPortTypeOperation addFault(String name, WsdlMessage messageRef) {
        QName q = buildQnameForMessage(messageRef);
        WsdlPortTypeFault fault = new WsdlPortTypeFault(name, q);
        return addFault(fault);
    }
    
    private QName buildQnameForMessage(WsdlMessage messageRef) throws XmlValidationException {
        if (messageRef == null) throw new IllegalArgumentException();
        WsdlDefinitions msgWsdl = messageRef.getDefinitions();
        if (msgWsdl == null) {
            throw new XmlValidationException(
                "input message must be inside wsdl:definitions to be referenced in " + xmlCuteElName());
        }
        String ns = msgWsdl.getTargetNamespace();
        XmlNamespace localNs = getDefinitions().xml().lookupNamespaceByName(ns);
        if (localNs == null) {
            throw new XmlValidationException(
                "could not find in wsdl:definitions namespace declaration for '" + ns
                + "' (context " + xmlCuteElName()+")");
        }
        String prefix = localNs.getPrefix();
        if (prefix == null) {
            throw new XmlValidationException(
                "cwsdl:definitions namespace declaration for '" + ns
                + "' is missing prefix required for QName references (context " + xmlCuteElName()+")");
        }
        String name = messageRef.getName();
        QName q = new QName(ns, name, prefix);
        return q;
    }
    
    
    public void xmlValidate() throws XmlValidationException {
        super.xmlValidate();
        getDefinitions();
        if (getInput() != null) {
            getInput().xmlValidate();
        }
        if (getOutput() != null) {
            getOutput().xmlValidate();
        }
    }
    
}


/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (C) 2004 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/





