/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004-2005 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: WsdlXmlViewBase.java,v 1.4 2006/04/26 23:22:18 aslom Exp $
 */
package xsul5.wsdl;

import javax.xml.namespace.QName;
import org.xmlpull.infoset.XmlAttribute;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.XmlNamespace;
import org.xmlpull.infoset.ext.XmlQNameAttribute;
import org.xmlpull.infoset.view.TypedXmlElementView;
import org.xmlpull.infoset.view.XmlValidationException;
import xsul5.XmlConstants;
import xsul5.XmlValidation;

/**
 * Abstract base class for all
 */
public abstract class WsdlXmlViewBase extends TypedXmlElementView implements XmlValidation {
    protected final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    public final static XmlNamespace NS = builder.newNamespace(
        "wsdl", "http://schemas.xmlsoap.org/wsdl/");
    private final String typeName;
    
    public WsdlXmlViewBase(String typeName, XmlElement target) {
        super(target);
        this.typeName = typeName;
    }
    
    protected WsdlXmlViewBase(XmlNamespace ns, String typeName) {
        super(builder.newFragment(ns, typeName));
        xml().addView(this);
        this.typeName = typeName;
    }
    
    public WsdlXmlViewBase(String typeName) {
        this(NS, typeName);
    }
        
    public XmlNamespace xmlTypeNs() { return NS; }
    public String xmlTypeName() { return typeName; }
    
    public WsdlDocumentation getDocumentation() {
        XmlElement docEl = xml().element(WsdlDocumentation.NS, WsdlDocumentation.NAME);
        if (docEl != null) {
            return docEl.viewAs(WsdlDocumentation.class);
        } else {
            return null;
        }
    }
    
    public void setDocumentation(WsdlDocumentation el) {
        xml().replaceLikeElementsWith(el.xml());
    }
    
    protected QName getMyQName(String tns, String name) {
        XmlNamespace ns = xml().lookupNamespaceByName(tns);
        if(ns == null) {
            throw new XmlValidationException(
                "missing declaration of namesapce '"+tns+" accessible form "+xmlCuteElName());
        }
        //XmlNamespace ns = getDefinitions().xml().getNamespace();
        QName qn = new QName(ns.getName(), name, ns.getPrefix());
        return qn;
    }
    
    protected void checkNamespaceAndName() throws XmlValidationException {
        String name = xml.getName();
        if (typeName != null && !name.equals(typeName)) {
            throw new XmlValidationException("expected " + typeName + " but got " + name);
        }
        XmlNamespace ns = xml.getNamespace();
        if (!xmlTypeNs().equals(ns)) {
            throw new XmlValidationException(
                "element must be in " + NS +
                " and not " + ns.getName());
        }
    }
    
    public String xmlString() {
        String s = builder.serializeToString(xml);
        return s;
    }
    
    public String xmlStringPretty() {
        String s = builder.serializeToStringPretty(xml);
        return s;
    }
    
    public String toString() {
        return xmlString();
    }
    
    
}

/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (c) 2004-2005 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/


