/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: PartnerLinkType.java,v 1.2 2006/05/01 22:14:48 aslom Exp $
 */

package xsul5.wsdl.plnk;


import javax.xml.namespace.QName;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.XmlNamespace;
import org.xmlpull.infoset.view.XmlValidationException;
import xsul5.MLogger;
import xsul5.XmlConstants;
import xsul5.wsdl.WsdlDefinitions;
import xsul5.wsdl.WsdlMessagePart;
import xsul5.wsdl.WsdlXmlViewBase;


//OLD "http://schemas.xmlsoap.org/ws/2003/05/partner-link/"
//NEW http://schemas.xmlsoap.org/ws/2004/03/partner-link/"

/**
 *
 * @version $Revision: 1.2 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class PartnerLinkType extends WsdlXmlViewBase {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;
    public final static XmlNamespace NS = builder.newNamespace(
        "plnk", "http://schemas.xmlsoap.org/ws/2004/03/partner-link/");
    public final static String TYPE_NAME = "partnerLinkType";
    
    public PartnerLinkType(String name) {
        super(NS, TYPE_NAME);
        setName(name);
    }
    
    public PartnerLinkType(XmlElement target) {
        super(TYPE_NAME, target);
    }
    
    public QName getQName() {
        return getMyQName(getDefinitions().getTargetNamespace(), getName());
    }
    
    public XmlNamespace xmlTypeNs() { return NS; }
    
    public WsdlDefinitions getDefinitions() {
        XmlElement defsEl = (XmlElement)xml().getParent();
        return defsEl.viewAs(WsdlDefinitions.class);
    }
    
    public void setName(String partName) throws XmlValidationException {
        xmlSetRequiredAttributeNcNameValue("name", partName);
    }
    
    public String getName() throws XmlValidationException {
        return xmlRequireAttributeNcNameValue("name");
    }
    
    public Iterable<PartnerLinkRole> roles() {
        return xml().elements(PartnerLinkRole.NS, PartnerLinkRole.TYPE_NAME, PartnerLinkRole.class);
    }

    public PartnerLinkRole getRole(String name) {
        for(PartnerLinkRole role : roles()) {
            if(name.equals(role.getName())) {
                return role;
            }
        }
        return null;
    }
    
    public PartnerLinkType addRole(PartnerLinkRole role) {
        xml().addElement(role.xml());
        return this;
    }

    public PartnerLinkType addRole(String name, QName portTypeQname) {
        PartnerLinkRole prole = new PartnerLinkRole(name, portTypeQname);
        xml().addElement(prole.xml());
        return this;
    }
        
    public void xmlValidate() throws XmlValidationException {
        super.xmlValidate();
        getName();
        for (PartnerLinkRole role: roles()) {
            role.xmlValidate();
        }
    }
    
    public String toString() {
        return builder.serializeToString(this) ;
    }
}


/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (C) 2004-2005 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/





