/* -*- mode: Java; c-basic-offset: 4; indent-tabs-mode: nil; -*-  //------100-columns-wide------>|*/
/*
 * Copyright (c) 2004 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the licence.
 *
 * $Id: WsdlSoapBinding.java,v 1.4 2006/04/28 20:58:47 aslom Exp $
 */

package xsul5.wsdl.soap;

import java.net.URI;
import java.net.URISyntaxException;
import javax.xml.namespace.QName;
import org.xmlpull.infoset.XmlElement;
import org.xmlpull.infoset.XmlInfosetBuilder;
import org.xmlpull.infoset.XmlNamespace;
import org.xmlpull.infoset.view.XmlValidationException;
import xsul5.MLogger;
import xsul5.XmlConstants;
import xsul5.wsdl.WsdlBinding;

/**
 * Implementation of WSLD 1.1 SOAP Bindings for services/port[/soap:address]
 *
 * @version $Revision: 1.4 $
 * @author <a href="http://www.extreme.indiana.edu/~aslom/">Aleksander Slominski</a>
 */
public class WsdlSoapBinding extends WsdlBinding {
    private final static MLogger logger = MLogger.getLogger();
    private final static XmlInfosetBuilder builder = XmlConstants.BUILDER;

    public enum STYLE {rpc, document}
    public final static URI HTTP_TRANSPORT = URI.create("http://schemas.xmlsoap.org/soap/http");
    //public final static XmlNamespace WSDL_SOAP_NS = builder.newNamespace(
    //    "soap", "http://schemas.xmlsoap.org/wsdl/soap/");
    
    public WsdlSoapBinding(String name, QName bindingQName) {
        super(name, bindingQName);
        setStyle(STYLE.document);
    }
    
    public WsdlSoapBinding(XmlElement target) {
        super(target);
    }
    
    public XmlElement getSoapBindingElement(boolean createIfDoesNotExist) {
        XmlElement soapBindingEl = xml().element(WsdlSoapUtil.WSDL_SOAP_NS, TYPE_NAME);
        if (soapBindingEl == null) {
            if (createIfDoesNotExist) {
                soapBindingEl = xml().newElement(WsdlSoapUtil.WSDL_SOAP_NS, TYPE_NAME);
                // make sure <soap:binding is first element - this looks nice ...
                xml().insertChild(0, soapBindingEl);
            }
        }
        return soapBindingEl;
    }
    
    public void setStyle(STYLE style) {
        String s = style.toString();
        getSoapBindingElement(true).setAttributeValue("style", s);
    }
    
    public STYLE getStyle() {
        String s = getSoapBindingElement(true).requiredAttributeValue("style");
        return STYLE.valueOf(s);
    }
    
    
    public void setTransport(URI transport) throws XmlValidationException {
        if (transport == null) throw new XmlValidationException("transport can not be null");
        getSoapBindingElement(true).setAttributeValue("transport", transport.toString());
    }
    
    public URI getTransport() {
        String s = getSoapBindingElement(true).requiredAttributeValue("transport");
        try {
            return new URI(s);
        } catch (URISyntaxException e) {
            throw new XmlValidationException(
                "could not convert transport value '" + s + " to URI", e);
        }
        
    }
}


/*
* Indiana University Extreme! Lab Software License, Version 1.2
*
* Copyright (C) 2004 The Trustees of Indiana University.
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are
* met:
*
* 1) All redistributions of source code must retain the above
*    copyright notice, the list of authors in the original source
*    code, this list of conditions and the disclaimer listed in this
*    license;
*
* 2) All redistributions in binary form must reproduce the above
*    copyright notice, this list of conditions and the disclaimer
*    listed in this license in the documentation and/or other
*    materials provided with the distribution;
*
* 3) Any documentation included with all redistributions must include
*    the following acknowledgement:
*
*      "This product includes software developed by the Indiana
*      University Extreme! Lab.  For further information please visit
*      http://www.extreme.indiana.edu/"
*
*    Alternatively, this acknowledgment may appear in the software
*    itself, and wherever such third-party acknowledgments normally
*    appear.
*
* 4) The name "Indiana University" or "Indiana University
*    Extreme! Lab" shall not be used to endorse or promote
*    products derived from this software without prior written
*    permission from Indiana University.  For written permission,
*    please contact http://www.extreme.indiana.edu/.
*
* 5) Products derived from this software may not use "Indiana
*    University" name nor may "Indiana University" appear in their name,
*    without prior written permission of the Indiana University.
*
* Indiana University provides no reassurances that the source code
* provided does not infringe the patent or any other intellectual
* property rights of any other entity.  Indiana University disclaims any
* liability to any recipient for claims brought by any other entity
* based on infringement of intellectual property rights or otherwise.
*
* LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH
* NO WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA
* UNIVERSITY GIVES NO WARRANTIES AND MAKES NO REPRESENTATION THAT
* SOFTWARE IS FREE OF INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR
* OTHER PROPRIETARY RIGHTS.  INDIANA UNIVERSITY MAKES NO WARRANTIES THAT
* SOFTWARE IS FREE FROM "BUGS", "VIRUSES", "TROJAN HORSES", "TRAP
* DOORS", "WORMS", OR OTHER HARMFUL CODE.  LICENSEE ASSUMES THE ENTIRE
* RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR ASSOCIATED MATERIALS,
* AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION GENERATED USING
* SOFTWARE.
*/

